"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const clone = require('clone');
const fs_extra_1 = require("fs-extra");
const error_1 = require("./error");
const logger_1 = require("./logger");
const log = (level, ...message) => logger_1.default(level, 'fileDb -', ...message);
const { internalError } = error_1.default('fileDb');
const outputFile = (path, data, options) => fs_extra_1.outputFile(global.storage + path, data, options);
const pathExists = path => fs_extra_1.pathExists(global.storage + path);
const readJSON = (path, options) => fs_extra_1.readJSON(global.storage + path, options);
const moveSync = (path, path2) => fs_extra_1.moveSync(global.storage + path, global.storage + path2);
class FileDb {
    constructor(path, initialState) {
        this.content = initialState;
        this.path = path;
        const numberedKeys = Object.keys(initialState)
            .map(key => +key)
            .filter(n => !isNaN(n));
        this.highestIndex = numberedKeys.length ? Math.max(...numberedKeys) : 0;
    }
    list() {
        log(8, `listed all elements from db ${this.path}`);
        return clone(Object.values(this.content));
    }
    get(id) {
        log(8, `read element ${id} from db ${this.path}`);
        return clone(this.content[id]);
    }
    find(key, value) {
        log(8, `searched for ${value} in column ${key} from db ${this.path}`);
        return clone(Object.values(this.content).filter(elem => elem[key] === value));
    }
    async set(id, value) {
        this.content[id] = clone(value);
        await write(this.content, this.path);
        log(8, `updated element ${id} from db ${this.path}`);
        return clone(value);
    }
    async setMultiple(map) {
        this.content = {
            ...this.content,
            ...clone(map),
        };
        await write(this.content, this.path);
        log(8, `updated elements ${Object.keys(map).join(', ')} from db ${this.path}`);
        return clone(map);
    }
    get nextIndex() {
        this.highestIndex++;
        return this.highestIndex.toString();
    }
    async delete(id) {
        delete this.content[id];
        await write(this.content, this.path);
        log(8, `deleted element ${id} from db ${this.path}`);
        return id;
    }
}
let writeDebounce;
let debounceContent;
function write(content, path) {
    if (writeDebounce) {
        debounceContent = content;
    }
    else {
        debounceContent = content;
        writeDebounce = new Promise(resolve => setTimeout(async () => {
            const data = JSON.stringify(debounceContent, null, 2);
            debounceContent = undefined;
            writeDebounce = undefined;
            await outputFile(path, data, 'utf8');
            resolve();
        }, 50));
    }
    return writeDebounce;
}
async function initDb(path, initialState = {}) {
    if (await pathExists(path)) {
        try {
            initialState = await readJSON(path, 'utf8');
        }
        catch (error) {
            internalError(1, 'error opening the stored db')(error);
            moveSync(path, path.split('.')[0] + '_backup_' + Date.now() + '.json');
        }
    }
    return new FileDb(path, initialState || {});
}
exports.default = initDb;
